! ESA (C) 2000-2018 
!
! This file is part of ESA's XMM-Newton Scientific Analysis System (SAS).
!
!    SAS is free software: you can redistribute it and/or modify
!    it under the terms of the GNU General Public License as published by
!    the Free Software Foundation, either version 3 of the License, or
!    (at your option) any later version.
!
!    SAS is distributed in the hope that it will be useful,
!    but WITHOUT ANY WARRANTY; without even the implied warranty of
!    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
!    GNU General Public License for more details.
!
!    You should have received a copy of the GNU General Public License
!    along with SAS.  If not, see <http://www.gnu.org/licenses/>.
program checkoutput

   use dal
   use errorhandling

   implicit none

   type(DataSetT) IIDataSetH
   type(ArrayT) IIArrayH
   real(kind=single), dimension(:,:), pointer :: image
   integer nDim
   integer(kind=int32), dimension(2) :: lb
   integer(kind=int32), dimension(2) :: ub
   real(kind=single), allocatable, dimension(:, :) :: tmp
   integer(kind=int32) allocErr
   real(kind=single) mean
   real(kind=single) variance
   real(kind=single) sd

   IIDataSetH = dataSet("testimage.fts", read)
   IIArrayH = array(IIDataSetH, "PRIMARY",read)
   image => real32Array2Data(IIArrayH)

   nDim = numberofDimensions(IIArrayH)

   if (nDim /=2) then
      call error("notImage", "Data in PRIMARY array is not an image")
   end if

   lb = lbound(image)
   ub = ubound(image)

   allocate (tmp(lb(1):ub(1), lb(2):ub(2)), stat=allocErr)
   if (allocErr /= 0) then
      call fatal("allocError", "Can't allocate memory for internal array.")
   end if

   ! 1999-11-17: GV@SOC added the real() below. Compiler bug??
   mean = sum(image) / real(size(image))
   tmp = (image - mean)**2
   variance = sum(tmp) / (size(image) - 1)
   sd = sqrt(variance)

   call release(IIArrayH)
   call release(IIDataSetH)

   if(allocated(tmp)) then
      deallocate(tmp ,stat=allocErr)
      if(allocErr /= 0) then
         call fatal("allocError", "Can't allocate memory for internal array.")
      end if
   end if

!   if (sd .gt. 10.0e-4) then
   if (sd .gt. 100.0) then
      write(*,*)'sd = ',sd
      call fatal("failed", "Standard deviation too large.")
   end if

end program







